@@ -1,17 +1,14 @@
-
 import base64
 import logging
 import re
-
 from collections import defaultdict
 from hashlib import sha1
+from urllib.parse import urlparse
 
-from streamlink.compat import urlparse
 from streamlink.exceptions import PluginError
-from streamlink.plugin import Plugin, PluginArguments, PluginArgument
+from streamlink.plugin import Plugin, PluginArgument, PluginArguments
 from streamlink.plugin.api import validate
-from streamlink.stream import HDSStream
-from streamlink.stream import HLSStream
+from streamlink.stream import HDSStream, HLSStream
 from streamlink.stream.dash import DASHStream
 from streamlink.utils import parse_json
 
@@ -34,7 +31,8 @@ class BBCiPlayer(Plugin):
     state_re = re.compile(r'window.__IPLAYER_REDUX_STATE__\s*=\s*({.*});')
     account_locals_re = re.compile(r'window.bbcAccount.locals\s*=\s*({.*?});')
     hash = base64.b64decode(b"N2RmZjc2NzFkMGM2OTdmZWRiMWQ5MDVkOWExMjE3MTk5MzhiOTJiZg==")
-    api_url = "https://open.live.bbc.co.uk/mediaselector/6/select/version/2.0/mediaset/{platform}/vpid/{vpid}/format/json/atk/{vpid_hash}/asn/1/"
+    api_url = "https://open.live.bbc.co.uk/mediaselector/6/select/version/2.0/mediaset/" \
+              "{platform}/vpid/{vpid}/format/json/atk/{vpid_hash}/asn/1/"
     platforms = ("pc", "iptv-all")
     session_url = "https://session.bbc.com/session"
     auth_url = "https://account.bbc.com/signin"
@@ -102,7 +100,7 @@ class BBCiPlayer(Plugin):
         :return: Video Packet ID for a Programme in iPlayer
         :rtype: string
         """
-        log.debug("Looking for vpid on {0}", url)
+        log.debug(f"Looking for vpid on {url}")
         # Use pre-fetched page if available
         res = res or self.session.http.get(url)
         m = self.mediator_re.search(res.text)
@@ -125,31 +123,25 @@ class BBCiPlayer(Plugin):
         for platform in self.platforms:
             url = self.api_url.format(vpid=vpid, vpid_hash=self._hash_vpid(vpid),
                                       platform=platform)
-            log.debug("Info API request: {0}", url)
+            log.debug(f"Info API request: {url}")
             medias = self.session.http.get(url, schema=self.mediaselector_schema)
             for media in medias:
                 for connection in media["connection"]:
                     urls[connection.get("transferFormat")].add(connection["href"])
 
         for stream_type, urls in urls.items():
-            log.debug("{0} {1} streams", len(urls), stream_type)
+            log.debug(f"{len(urls)} {stream_type} streams")
             for url in list(urls):
                 try:
                     if stream_type == "hds":
-                        for s in HDSStream.parse_manifest(self.session,
-                                                          url).items():
-                            yield s
+                        yield from HDSStream.parse_manifest(self.session, url).items()
                     if stream_type == "hls":
-                        for s in HLSStream.parse_variant_playlist(self.session,
-                                                                  url).items():
-                            yield s
+                        yield from HLSStream.parse_variant_playlist(self.session, url).items()
                     if stream_type == "dash":
-                        for s in DASHStream.parse_manifest(self.session,
-                                                           url).items():
-                            yield s
-                    log.debug("  OK:   {0}", url)
+                        yield from DASHStream.parse_manifest(self.session, url).items()
+                    log.debug(f"  OK:   {url}")
                 except Exception:
-                    log.debug("  FAIL: {0}", url)
+                    log.debug(f"  FAIL: {url}")
 
     def login(self, ptrt_url):
         """
@@ -206,34 +198,29 @@ class BBCiPlayer(Plugin):
         channel_name = m.group("channel_name")
 
         if episode_id:
-            log.debug("Loading streams for episode: {0}", episode_id)
+            log.debug(f"Loading streams for episode: {episode_id}")
             vpid = self.find_vpid(self.url)
             if vpid:
-                log.debug("Found VPID: {0}", vpid)
-                for s in self.mediaselector(vpid):
-                    yield s
+                log.debug(f"Found VPID: {vpid}")
+                yield from self.mediaselector(vpid)
             else:
-                log.error("Could not find VPID for episode {0}",
-                          episode_id)
+                log.error(f"Could not find VPID for episode {episode_id}")
         elif channel_name:
-            log.debug("Loading stream for live channel: {0}", channel_name)
+            log.debug(f"Loading stream for live channel: {channel_name}")
             if self.get_option("hd"):
                 tvip = self.find_tvip(self.url, master=True) + "_hd"
                 if tvip:
-                    log.debug("Trying HD stream {0}...", tvip)
+                    log.debug(f"Trying HD stream {tvip}...")
                     try:
-                        for s in self.mediaselector(tvip):
-                            yield s
+                        yield from self.mediaselector(tvip)
                     except PluginError:
-                        log.error(
-                            "Failed to get HD streams, falling back to SD")
+                        log.error("Failed to get HD streams, falling back to SD")
                     else:
                         return
             tvip = self.find_tvip(self.url)
             if tvip:
-                log.debug("Found TVIP: {0}", tvip)
-                for s in self.mediaselector(tvip):
-                    yield s
+                log.debug(f"Found TVIP: {tvip}")
+                yield from self.mediaselector(tvip)
 
 
 __plugin__ = BBCiPlayer
