@@ -1,21 +1,19 @@
 import argparse
+import numbers
 import re
 from string import printable
 from textwrap import dedent
 
-from streamlink import logger
-from streamlink.utils.times import hours_minutes_seconds
-from .constants import (
-    LIVESTREAMER_VERSION, STREAM_PASSTHROUGH, DEFAULT_PLAYER_ARGUMENTS
+from streamlink import __version__ as streamlink_version, logger
+from streamlink.utils.args import (
+    boolean, comma_list, comma_list_filter, filesize, keyvalue, num
 )
-from .utils import find_default_player
+from streamlink.utils.times import hours_minutes_seconds
+from streamlink_cli.constants import (
+    DEFAULT_STREAM_METADATA, PLAYER_ARGS_INPUT_DEFAULT, PLAYER_ARGS_INPUT_FALLBACK, STREAM_PASSTHROUGH, SUPPORTED_PLAYERS
+)
+from streamlink_cli.utils import find_default_player
 
-_filesize_re = re.compile(r"""
-    (?P<size>\d+(\.\d+)?)
-    (?P<modifier>[Kk]|[Mm])?
-    (?:[Bb])?
-""", re.VERBOSE)
-_keyvalue_re = re.compile(r"(?P<key>[^=]+)\s*=\s*(?P<value>.*)")
 _printable_re = re.compile(r"[{0}]".format(printable))
 _option_re = re.compile(r"""
     (?P<name>[A-z-]+) # A option name, valid characters are A to z and dash.
@@ -43,9 +41,68 @@ class ArgumentParser(argparse.ArgumentParser):
 
         name, value = option.group("name", "value")
         if name and value:
-            yield "--{0}={1}".format(name, value)
+            yield f"--{name}={value}"
         elif name:
-            yield "--{0}".format(name)
+            yield f"--{name}"
+
+    def _match_argument(self, action, arg_strings_pattern):
+        # - https://github.com/streamlink/streamlink/issues/971
+        # - https://bugs.python.org/issue9334
+
+        # match the pattern for this action to the arg strings
+        nargs_pattern = self._get_nargs_pattern(action)
+        match = argparse._re.match(nargs_pattern, arg_strings_pattern)
+
+        # if no match, see if we can emulate optparse and return the
+        # required number of arguments regardless of their values
+        if match is None:
+            nargs = action.nargs if action.nargs is not None else 1
+            if isinstance(nargs, numbers.Number) and len(arg_strings_pattern) >= nargs:
+                return nargs
+
+        # raise an exception if we weren't able to find a match
+        if match is None:
+            nargs_errors = {
+                None: argparse._('expected one argument'),
+                argparse.OPTIONAL: argparse._('expected at most one argument'),
+                argparse.ONE_OR_MORE: argparse._('expected at least one argument'),
+            }
+            default = argparse.ngettext('expected %s argument',
+                                        'expected %s arguments',
+                                        action.nargs) % action.nargs
+            msg = nargs_errors.get(action.nargs, default)
+            raise argparse.ArgumentError(action, msg)
+
+        # return the number of arguments matched
+        return len(match.group(1))
+
+    # fix `--help` not including nested argument groups
+    def format_help(self):
+        formatter = self._get_formatter()
+
+        # usage
+        formatter.add_usage(self.usage, self._actions,
+                            self._mutually_exclusive_groups)
+
+        # description
+        formatter.add_text(self.description)
+
+        def format_group(group):
+            # positionals, optionals and user-defined groups
+            for action_group in group._action_groups:
+                formatter.start_section(action_group.title)
+                formatter.add_text(action_group.description)
+                formatter.add_arguments(action_group._group_actions)
+                format_group(action_group)
+                formatter.end_section()
+
+        format_group(self)
+
+        # epilog
+        formatter.add_text(self.epilog)
+
+        # determine help from format above
+        return formatter.format_help()
 
 
 class HelpFormatter(argparse.RawDescriptionHelpFormatter):
@@ -68,78 +125,6 @@ class HelpFormatter(argparse.RawDescriptionHelpFormatter):
         return text.splitlines()
 
 
-def comma_list(values):
-    return [val.strip() for val in values.split(",")]
-
-
-def comma_list_filter(acceptable):
-    def func(p):
-        values = comma_list(p)
-        return list(filter(lambda v: v in acceptable, values))
-
-    return func
-
-
-def num(type, min=None, max=None):
-    def func(value):
-        value = type(value)
-
-        if min is not None and not (value > min):
-            raise argparse.ArgumentTypeError(
-                "{0} value must be more than {1} but is {2}".format(
-                    type.__name__, min, value
-                )
-            )
-
-        if max is not None and not (value <= max):
-            raise argparse.ArgumentTypeError(
-                "{0} value must be at most {1} but is {2}".format(
-                    type.__name__, max, value
-                )
-            )
-
-        return value
-
-    func.__name__ = type.__name__
-
-    return func
-
-
-def filesize(value):
-    match = _filesize_re.match(value)
-    if not match:
-        raise ValueError
-
-    size = float(match.group("size"))
-    if not size:
-        raise ValueError
-
-    modifier = match.group("modifier")
-    if modifier in ("M", "m"):
-        size *= 1024 * 1024
-    elif modifier in ("K", "k"):
-        size *= 1024
-
-    return num(int, min=0)(size)
-
-
-def keyvalue(value):
-    match = _keyvalue_re.match(value)
-    if not match:
-        raise ValueError
-
-    return match.group("key", "value")
-
-
-def boolean(value):
-    truths = ["yes", "1", "true", "on"]
-    falses = ["no", "0", "false", "off"]
-    if value.lower() not in truths + falses:
-        raise argparse.ArgumentTypeError("{0} was not one of {{{1}}}".format(value, ', '.join(truths + falses)))
-
-    return value.lower() in truths
-
-
 def build_parser():
     parser = ArgumentParser(
         prog="streamlink",
@@ -182,7 +167,7 @@ def build_parser():
         help="""
         Stream to play.
 
-        Use "best" or "worst" for selecting the highest or lowest available
+        Use ``best`` or ``worst`` for selecting the highest or lowest available
         quality.
 
         Fallback streams can be specified by using a comma-separated list:
@@ -205,7 +190,7 @@ def build_parser():
     general.add_argument(
         "-V", "--version",
         action="version",
-        version="%(prog)s {0}".format(LIVESTREAMER_VERSION),
+        version=f"%(prog)s {streamlink_version}",
         help="""
         Show version number and exit.
         """
@@ -319,11 +304,25 @@ def build_parser():
         """
     )
     general.add_argument(
-        "--twitch-oauth-authenticate",
+        "--interface",
+        type=str,
+        metavar="INTERFACE",
+        help="""
+        Set the network interface.
+        """
+    )
+    general.add_argument(
+        "-4", "--ipv4",
         action="store_true",
         help="""
-        Open a web browser where you can grant Streamlink access to your Twitch
-        account which creates a token for use with --twitch-oauth-token.
+        Resolve address names to IPv4 only. This option overrides :option:`-6`.
+        """
+    )
+    general.add_argument(
+        "-6", "--ipv6",
+        action="store_true",
+        help="""
+        Resolve address names to IPv6 only. This option overrides :option:`-4`.
         """
     )
 
@@ -363,33 +362,36 @@ def build_parser():
     player.add_argument(
         "-a", "--player-args",
         metavar="ARGUMENTS",
-        default=DEFAULT_PLAYER_ARGUMENTS,
-        help="""
+        default="",
+        help=f"""
         This option allows you to customize the default arguments which are put
         together with the value of --player to create a command to execute.
-        Unlike the --player parameter, custom player arguments will not be logged.
 
-        This value can contain formatting variables surrounded by curly braces,
+        It's usually enough to only use --player instead of this unless you need
+        to add arguments after the player's input argument or if you don't want
+        any of the player arguments to be logged.
+
+        The value can contain formatting variables surrounded by curly braces,
         {{ and }}. If you need to include a brace character, it can be escaped
         by doubling, e.g. {{{{ and }}}}.
 
         Formatting variables available:
 
-        filename
-            This is the filename that the player will use. It's usually "-"
-            (stdin), but can also be a URL or a file depending on the options
-            used.
+        {{{PLAYER_ARGS_INPUT_DEFAULT}}}
+            This is the input that the player will use. For standard input (stdin),
+            it is ``-``, but it can also be a URL, depending on the options used.
 
-        It's usually enough to use --player instead of this unless you need to
-        add arguments after the filename.
-
-        Default is "{0}".
+        {{{PLAYER_ARGS_INPUT_FALLBACK}}}
+            The old fallback variable name with the same functionality.
 
         Example:
 
-          %(prog)s -p vlc -a "--play-and-exit {{filename}}" <url> [stream]
+          %(prog)s -p vlc -a "--play-and-exit {{{PLAYER_ARGS_INPUT_DEFAULT}}}" <url> [stream]
 
-        """.format(DEFAULT_PLAYER_ARGUMENTS)
+        Note: When neither of the variables are found, ``{{{PLAYER_ARGS_INPUT_DEFAULT}}}``
+        will be appended to the whole parameter value, to ensure that the player
+        always receives an input argument.
+        """
     )
     player.add_argument(
         "-v", "--verbose-player",
@@ -481,6 +483,78 @@ def build_parser():
         This option will instead let the player decide when to exit.
         """
     )
+    player.add_argument(
+        "-t", "--title",
+        metavar="TITLE",
+        help="""
+        This option allows you to supply a title to be displayed in the
+        title bar of the window that the video player is launched in.
+
+        This value can contain formatting variables surrounded by curly braces,
+        {{ and }}. If you need to include a brace character, it can be escaped
+        by doubling, e.g. {{{{ and }}}}.
+
+        This option is only supported for the following players: {0}.
+
+        VLC specific information:
+            VLC has certain codes you can use inside your title.
+            These are accessible inside --title by using a backslash
+            before the dollar sign VLC uses to denote a format character.
+
+            e.g. to put the current date in your VLC window title,
+            the string "\\$A" could be inserted inside your --title string.
+
+            A full list of the format codes VLC uses is available here:
+            https://wiki.videolan.org/Documentation:Format_String/
+
+        mpv specific information:
+            mpv has certain codes you can use inside your title.
+            These are accessible inside --title by using a backslash
+            before the dollar sign mpv uses to denote a format character.
+
+            e.g. to put the current version of mpv running inside your
+            mpv window title, the string "\\${{{{mpv-version}}}}" could be
+            inserted inside your --title string.
+
+            A full list of the format codes mpv uses is available here:
+            https://mpv.io/manual/stable/#property-list
+
+        Formatting variables available to use in --title:
+
+        {{title}}
+            If available, this is the title of the stream.
+            Otherwise, it is the string "{1}"
+
+        {{author}}
+            If available, this is the author of the stream.
+            Otherwise, it is the string "{2}"
+
+        {{category}}
+            If available, this is the category the stream has been placed into.
+
+            - For Twitch, this is the game being played
+            - For YouTube, it's the category e.g. Gaming, Sports, Music...
+
+            Otherwise, it is the string "{3}"
+
+        {{game}}
+            This is just a synonym for {{category}} which may make more sense for
+            gaming oriented platforms. "Game being played" is a way to categorize
+            the stream, so it doesn't need its own separate handling.
+
+        {{url}}
+            URL of the stream.
+
+        Examples:
+
+            %(prog)s -p vlc --title "{{title}} -!- {{author}} -!- {{category}} \\$A" <url> [stream]
+            %(prog)s -p mpv --title "{{title}} -- {{author}} -- {{category}} -- (\\${{{{mpv-version}}}})" <url> [stream]
+
+        """.format(', '.join(sorted(SUPPORTED_PLAYERS.keys())),
+                   DEFAULT_STREAM_METADATA['title'],
+                   DEFAULT_STREAM_METADATA['author'],
+                   DEFAULT_STREAM_METADATA['category'])
+    )
 
     output = parser.add_argument_group("File output options")
     output.add_argument(
@@ -496,7 +570,15 @@ def build_parser():
         "-f", "--force",
         action="store_true",
         help="""
-        When using -o, always write to file even if it already exists.
+        When using -o or -r, always write to file even if it already exists.
+        """
+    )
+    output.add_argument(
+        "--force-progress",
+        action="store_true",
+        help="""
+        When using -o or -r,
+        show the download progress bar even if there is no terminal.
         """
     )
     output.add_argument(
@@ -506,6 +588,24 @@ def build_parser():
         Write stream data to stdout instead of playing it.
         """
     )
+    output.add_argument(
+        "-r", "--record",
+        metavar="FILENAME",
+        help="""
+        Open the stream in the player, while at the same time writing it to FILENAME.
+
+        You will be prompted if the file already exists.
+        """
+    )
+    output.add_argument(
+        "-R", "--record-and-pipe",
+        metavar="FILENAME",
+        help="""
+        Write stream data to stdout, while at the same time writing it to FILENAME.
+
+        You will be prompted if the file already exists.
+        """
+    )
 
     stream = parser.add_argument_group("Stream options")
     stream.add_argument(
@@ -529,7 +629,7 @@ def build_parser():
         help="""
         Stream to play.
 
-        Use "best" or "worst" for selecting the highest or lowest available
+        Use ``best`` or ``worst`` for selecting the highest or lowest available
         quality.
 
         Fallback streams can be specified by using a comma-separated list:
@@ -540,6 +640,13 @@ def build_parser():
         and can be useful if set in a config file.
         """
     )
+    stream.add_argument(
+        "--stream-url",
+        action="store_true",
+        help="""
+        If possible, translate the resolved stream to a URL and print it.
+        """
+    )
     stream.add_argument(
         "--retry-streams",
         metavar="DELAY",
@@ -596,13 +703,17 @@ def build_parser():
         metavar="STREAMS",
         type=comma_list,
         help="""
-        Fine tune best/worst synonyms by excluding unwanted streams.
+        Fine tune the ``best`` and ``worst`` stream name synonyms by excluding unwanted streams.
+
+        If all of the available streams get excluded, ``best`` and ``worst`` will become
+        inaccessible and new special stream synonyms ``best-unfiltered`` and ``worst-unfiltered``
+        can be used as a fallback selection method.
 
         Uses a filter expression in the format:
 
           [operator]<value>
 
-        Valid operators are >, >=, < and <=. If no operator is specified then
+        Valid operators are ``>``, ``>=``, ``<`` and ``<=``. If no operator is specified then
         equality is tested.
 
         For example this will exclude streams ranked higher than "480p":
@@ -684,6 +795,13 @@ def build_parser():
         Default is 3.
         """
     )
+    transport.add_argument(
+        "--hls-segment-stream-data",
+        action="store_true",
+        help="""
+        Immediately write segment data into output buffer while downloading.
+        """
+    )
     transport.add_argument(
         "--hls-segment-attempts",
         type=num(int, min=0),
@@ -706,6 +824,20 @@ def build_parser():
         Default is 3.
         """
     )
+    transport.add_argument(
+        "--hls-playlist-reload-time",
+        metavar="TIME",
+        help="""
+        Set a custom HLS playlist reload time value, either in seconds
+        or by using one of the following keywords:
+
+            segment: The duration of the last segment in the current playlist
+            live-edge: The sum of segment durations of the live edge value minus one
+            default: The playlist's target duration metadata
+
+        Default is default.
+        """
+    )
     transport.add_argument(
         "--hls-segment-threads",
         type=num(int, max=10),
@@ -731,16 +863,35 @@ def build_parser():
         metavar="NAMES",
         type=comma_list,
         help="""
-        A comma-delimited list of segment names that will not be fetched.
+        A comma-delimited list of segment names that will get filtered out.
 
         Example: --hls-segment-ignore-names 000,001,002
 
         This will ignore every segment that ends with 000.ts, 001.ts and 002.ts
 
         Default is None.
+        """
+    )
+    transport.add_argument(
+        "--hls-segment-key-uri",
+        metavar="URI",
+        type=str,
+        help="""
+        URI to segment encryption key. If no URI is specified, the URI contained
+        in the segments will be used.
 
-        Note: The --hls-timeout must be increased, to a time that is longer than
-        the ignored break.
+        URI can be templated using the following variables, which will be
+        replaced with its respective part from the source segment URI:
+
+          {url} {scheme} {netloc} {path} {query}
+
+        Examples:
+
+          --hls-segment-key-uri "https://example.com/hls/encryption_key"
+          --hls-segment-key-uri "{scheme}://1.2.3.4{path}{query}"
+          --hls-segment-key-uri "{scheme}://{netloc}/custom/path/to/key"
+
+        Default is None.
         """
     )
     transport.add_argument(
@@ -774,7 +925,7 @@ def build_parser():
     transport.add_argument(
         "--hls-start-offset",
         type=hours_minutes_seconds,
-        metavar="HH:MM:SS",
+        metavar="[HH:]MM:SS",
         default=None,
         help="""
         Amount of time to skip from the beginning of the stream. For live
@@ -785,7 +936,7 @@ def build_parser():
     transport.add_argument(
         "--hls-duration",
         type=hours_minutes_seconds,
-        metavar="HH:MM:SS",
+        metavar="[HH:]MM:SS",
         default=None,
         help="""
         Limit the playback duration, useful for watching segments of a stream.
@@ -835,7 +986,7 @@ def build_parser():
         processing (such as HDS) to avoid unnecessary background processing.
         """)
     transport.add_argument(
-        "--rtmp-proxy", "--rtmpdump-proxy",
+        "--rtmp-proxy",
         metavar="PROXY",
         help="""
         A SOCKS proxy that RTMP streams will use.
@@ -844,7 +995,7 @@ def build_parser():
         """
     )
     transport.add_argument(
-        "--rtmp-rtmpdump", "--rtmpdump", "-r",
+        "--rtmp-rtmpdump",
         metavar="FILENAME",
         help="""
         RTMPDump is used to access RTMP streams. You can specify the
@@ -853,6 +1004,7 @@ def build_parser():
         Example: "/usr/local/bin/rtmpdump"
         """
     )
+    transport.add_argument("--rtmpdump", help=argparse.SUPPRESS)
     transport.add_argument(
         "--rtmp-timeout",
         type=num(float, min=0),
@@ -916,14 +1068,7 @@ def build_parser():
         Default is 60.0.
         """)
     transport.add_argument(
-        "--stream-url",
-        action="store_true",
-        help="""
-        If possible, translate the stream to a URL and print it.
-        """
-    )
-    transport.add_argument(
-        "--subprocess-cmdline", "--cmdline", "-c",
+        "--subprocess-cmdline",
         action="store_true",
         help="""
         Print the command-line used internally to play the stream.
@@ -932,7 +1077,7 @@ def build_parser():
         """
     )
     transport.add_argument(
-        "--subprocess-errorlog", "--errorlog", "-e",
+        "--subprocess-errorlog",
         action="store_true",
         help="""
         Log possible errors from internal subprocesses to a temporary file. The
@@ -942,7 +1087,7 @@ def build_parser():
         """
     )
     transport.add_argument(
-        "--subprocess-errorlog-path", "--errorlog-path",
+        "--subprocess-errorlog-path",
         type=str,
         metavar="PATH",
         help="""
@@ -978,11 +1123,23 @@ def build_parser():
         Path to write the output from the ffmpeg console.
         """
     )
+    transport.add_argument(
+        "--ffmpeg-fout",
+        type=str,
+        metavar="OUTFORMAT",
+        help="""
+        When muxing streams, set the output format to OUTFORMAT.
+
+        Default is "matroska".
+
+        Example: "mpegts"
+        """
+    )
     transport.add_argument(
         "--ffmpeg-video-transcode",
         metavar="CODEC",
         help="""
-        When muxing streams transcode the video to this CODEC.
+        When muxing streams, transcode the video to CODEC.
 
         Default is "copy".
 
@@ -993,20 +1150,45 @@ def build_parser():
         "--ffmpeg-audio-transcode",
         metavar="CODEC",
         help="""
-        When muxing streams transcode the audio to this CODEC.
+        When muxing streams, transcode the audio to CODEC.
 
         Default is "copy".
 
         Example: "aac"
         """
     )
+    transport.add_argument(
+        "--ffmpeg-copyts",
+        action="store_true",
+        help="""
+        Forces the -copyts ffmpeg option and does not remove
+        the initial start time offset value.
+        """
+    )
+    transport.add_argument(
+        "--ffmpeg-start-at-zero",
+        action="store_true",
+        help="""
+        Enable the -start_at_zero ffmpeg option when using copyts.
+        """
+    )
+    transport.add_argument(
+        "--mux-subtitles",
+        action="store_true",
+        help="""
+        Automatically mux available subtitles into the output stream.
+
+        Needs to be supported by the used plugin.
+        """
+    )
 
     http = parser.add_argument_group("HTTP options")
     http.add_argument(
         "--http-proxy",
         metavar="HTTP_PROXY",
         help="""
-        A HTTP proxy to use for all HTTP requests.
+        A HTTP proxy to use for all HTTP requests, including WebSocket connections.
+        By default this proxy will be used for all HTTPS requests too.
 
         Example: "http://hostname:port/"
         """
@@ -1110,43 +1292,6 @@ def build_parser():
         """
     )
 
-    # Deprecated options
-    stream.add_argument(
-        "--best-stream-default",
-        action="store_true",
-        help=argparse.SUPPRESS
-    )
-    player.add_argument(
-        "-q", "--quiet-player",
-        action="store_true",
-        help=argparse.SUPPRESS
-    )
-    transport.add_argument(
-        "--hds-fragment-buffer",
-        type=int,
-        metavar="fragments",
-        help=argparse.SUPPRESS
-    )
-    http.add_argument(
-        "--http-cookies",
-        metavar="COOKIES",
-        help=argparse.SUPPRESS
-    )
-    http.add_argument(
-        "--http-headers",
-        metavar="HEADERS",
-        help=argparse.SUPPRESS
-    )
-    http.add_argument(
-        "--http-query-params",
-        metavar="PARAMS",
-        help=argparse.SUPPRESS
-    )
-    general.add_argument(
-        "--no-version-check",
-        action="store_true",
-        help=argparse.SUPPRESS
-    )
     return parser
 
 
